
/*
 * This file has been auto-generated from the introspection data available
 * in the at-spi2-core repository. The D-Bus procol is defined in this
 * repository, which can be found at:
 *
 * http://download.gnome.org/sources/at-spi2-core/0.1/
 *
 * DO NOT EDIT.
 */

#ifndef SPI_INTROSPECTION_DATA_H_
#define SPI_INTROSPECTION_DATA_H_


extern const char *spi_org_a11y_atspi_Accessible;

extern const char *spi_org_a11y_atspi_Action;

extern const char *spi_org_a11y_atspi_Application;

extern const char *spi_org_a11y_atspi_Collection;

extern const char *spi_org_a11y_atspi_Component;

extern const char *spi_org_a11y_atspi_Document;

extern const char *spi_org_a11y_atspi_Hypertext;

extern const char *spi_org_a11y_atspi_Hyperlink;

extern const char *spi_org_a11y_atspi_Image;

extern const char *spi_org_a11y_atspi_Selection;

extern const char *spi_org_a11y_atspi_Table;

extern const char *spi_org_a11y_atspi_Text;

extern const char *spi_org_a11y_atspi_EditableText;

extern const char *spi_org_a11y_atspi_Cache;

extern const char *spi_org_a11y_atspi_Value;

extern const char *spi_org_a11y_atspi_Registry;

extern const char *spi_org_a11y_atspi_DeviceEventController;

extern const char *spi_org_a11y_atspi_DeviceEventListener;


#endif /* SPI_INTROSPECTION_DATA_H_ */
